;; -*- lexical-binding: t; -*-

;;;; Blog stuff.
;;; Edit Blognir posts in html-mode.
(add-to-list 'auto-mode-alist '("/\\.data/entries/" . html-mode))

(defun elmord-new-blog-post (id)
  (interactive "sPost id: ")
  (let ((filename (concat "/home/www/internal/html/public_html/blog/.data/entries/"
                          (format-time-string "%Y%m%d")
                          "-" id)))
    (with-current-buffer (find-file filename)
      (insert (concat "Title: \n"
                      "Created: " (format-time-string "%Y-%m-%d %H:%M %z") "\n"
                      "Tags: \n\n"))
      (beginning-of-buffer)
      (end-of-line))))


(define-key ctl-z-map (kbd "b") 'elmord-new-blog-post)


(require 'sgml-mode)

(defmacro -> (&rest forms)
  (if (null (cdr forms))
      (car forms)
    (let ((first (car forms))
          (second (cadr forms))
          (rest (cddr forms)))
      `(-> (,(car second) ,first ,@(cdr second))
           ,@rest))))


(defmacro ->> (&rest forms)
  (if (null (cdr forms))
      (car forms)
    (let ((first (car forms))
          (second (cadr forms))
          (rest (cddr forms)))
      `(->> (,@second ,first)
           ,@rest))))

(defvar elmord-org-blog-post-id)

(defun elmord-org-export-blog-post ()
  (interactive)
  (let ((elmord-org-blog-post-id (car (split-string (file-name-base) "-"))))
    (->> (elmord-org-export-blog-post-initial)
         (elmord-org-export-blog-post-fixup)
         (elmord-org-export-blog-post-add-header)
         )))

(defun elmord-org-export-blog-post-initial ()
  (save-current-buffer
    (let ((org-html-toplevel-hlevel 3)
          (org-export-with-toc nil)
          (org-export-with-section-numbers nil))
      (org-html-export-as-html
       nil ;; async
       nil ;; subtree-p
       nil ;; visible-only
       t   ;; body-only
       nil ;; ext-plist
       )
      )))

(defun elmord-replace-all (regex replacement)
  (beginning-of-buffer)
  (while (re-search-forward regex nil t)
    (replace-match replacement)))

(defun elmord-org-export-blog-post-fixup (output)
  (with-current-buffer output
    (html-mode)
    (beginning-of-buffer)
    (when (search-forward-regexp (rx (and point (1+ "\n"))) nil t)
      (replace-match ""))
    (dolist (regex '("outline-[^\"]*" "text-[^\"]*"))
      (elmord-remove-tags-with-id regex))
    (dolist (replacement `(("<\\(h[1-6]\\) id=\"sec-[^\"]*\"" . "<\\1")
                           ("<p>\n" . "<p>")
                           ("</p>" . "")
                           ("\n\n\n*" . "\n\n")
                           ("<a id=\"fnr\\.\\([0-9]+\\)\" name=\"fnr.\\1\" class=\"footref\" href=\"#fn.\\1\">" .
                            ,(replace-regexp-in-string
                              "@" elmord-org-blog-post-id
                              "<a id=\"ref\\1-@\" name=\"ref\\1-@\" class=\"footref\" href=\"#note\\1-@\">"))
                           ("<a id=\"fn\\.\\([0-9]+\\)\" name=\"fn.\\1\" class=\"footnum\" href=\"#fnr.\\1\">" .
                            ,(replace-regexp-in-string
                              "@" elmord-org-blog-post-id
                              "<a id=\"note\\1-@\" name=\"note\\1-@\" class=\"footnum\" href=\"#ref\\1-@\">"))
                           ("<a href=\"https://\\?" . "<a href=\"?")
                           ))
      (elmord-replace-all (car replacement) (cdr replacement)))
    )
  output)

(defun elmord-remove-tags-with-id (regex)
  (beginning-of-buffer)
  (while (re-search-forward (concat "<[^>]* id=\"" regex "\"") nil t)
    (search-backward "<")
    (save-excursion
      (sgml-skip-tag-forward 1)
      (backward-kill-sexp))
    (kill-sexp)))


;; Partly based on https://emacs.stackexchange.com/questions/21459/programmatically-read-and-set-buffer-wide-org-mode-property
(defun elmord-org-get-buffer-properties ()
  (org-element-map
      (org-element-parse-buffer)
      'keyword
    (lambda (el)
      (cond
       ((equal (org-element-property :key el) "PROPERTY")
        (let ((value (org-element-property :value el)))
          (when (string-match "^\\([^[:space:]]*\\)[[:space:]]+\\(.*\\)" value)
            (cons (match-string 1 value)
                  (match-string 2 value)))))
       ((member (org-element-property :key el) '("TITLE" "DATE"))
        (cons (org-element-property :key el)
              (org-element-property :value el)))))))


(defun elmord-org-export-blog-post-add-header (output)
  (let ((data (elmord-org-get-buffer-properties)))
    (with-current-buffer output
      (beginning-of-buffer)
      (insert
       (format "Title: %s\n" (cdr (assoc "TITLE" data)))
       (format "Created: %s\n" (elmord-org-format-date (cdr (assoc "DATE" data))))
       (format "Tags: %s\n" (cdr (assoc "Tags:" data)))
       "\n"))))

(defun elmord-org-format-date (stamp)
  (format-time-string
   "%Y-%m-%d %H:%M %z"
   (apply 'encode-time (org-parse-time-string stamp))))





;;;;;;;;;;;;

(defun elmord-html-toggle-tag (tag)
  (let ((next-tag
         (save-excursion
           (re-search-forward "<\\([^> ]*\\)" nil t)
           (match-string 1))))
    (if (equal next-tag (concat "/" tag))
        (search-forward ">")
      (skeleton-insert '(nil "<" tag ">" _ "</" tag ">")))))

(defun elmord-html-toggle-italic ()
  (interactive)
  (elmord-html-toggle-tag "i"))

(eval-after-load 'html
  (defun elmord-html-eval-after-load ()
    (define-key html-mode-map (kbd "M-i") 'elmord-html-toggle-italic)))
